@@ -26,6 +26,7 @@
 #include "application.h"
 #include "atom.h"
 #include "bond.h"
+#include "brackets.h"
 #include "document.h"
 #include "electron.h"
 #include "fragment.h"
@@ -52,6 +53,7 @@
 #include "view.h"
 #include "window.h"
 #include "zoomdlg.h"
+#include <gcu/cmd-context.h>
 #include <gcu/filechooser.h>
 #include <gcu/loader.h>
 #include <gcu/loader-error.h>
@@ -59,7 +61,6 @@
 #include <openbabel/mol.h>
 #include <openbabel/reaction.h>
 #include <openbabel/obconversion.h>
-#include <clocale>
 #include <fstream>
 #include <cstring>
 #include <sys/stat.h>
@@ -260,6 +261,11 @@ static Object* CreateElectron ()
 	return new Electron (NULL, false);
 }
 
+static Object* CreateBrackets ()
+{
+	return new Brackets ();
+}
+
 bool	Application::m_bInit = false;
 bool	Application::m_Have_Ghemical = false;
 bool	Application::m_Have_InChI = false;
@@ -269,17 +275,18 @@ static void on_config_changed (GOConfNode *node, gchar const *key, Application *
 	app->OnConfigChanged (node, key);
 }
 
-Application::Application ():
-	gcu::Application ("GChemPaint", DATADIR, "gchempaint", "gchempaint")
+Application::Application (CmdContext *cc):
+	gcu::Application ("GChemPaint", DATADIR, "gchempaint", "gchempaint", cc)
 {
 	m_CurZ = 6;
 	m_pActiveDoc = NULL;
 	m_pActiveTool = NULL;
+	m_pActiveTarget = NULL;
 	m_NumWindow = 1;
 
 	if (!m_bInit) {
 		/* Initialize plugins manager */
-		gcu::Loader::Init ();
+		gcu::Loader::Init (this);
 		// Check for programs
 		char *result = NULL, *errors = NULL;
 		// check for ghemical
@@ -323,6 +330,7 @@ Application::Application ():
 		AddType ("fragment", CreateFragment, FragmentType);
 		ElectronType = AddType ("electron", CreateElectron);
 		Object::AddAlias (ElectronType, "electron-pair");
+		BracketsType = AddType ("brackets", CreateBrackets);
 		// Add rules
 		AddRule ("reaction", RuleMustContain, "reaction-step");
 		AddRule ("reaction-step", RuleMustContain, "reactant");
@@ -516,25 +524,22 @@ enum {
 	PIXBUF
 };
 
-bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, gcu::Document *Doc)
+bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, G_GNUC_UNUSED GtkWindow *window, gcu::Document *Doc)
 {
 	const gchar* ext;
 	Document *pDoc = static_cast<Document*> (Doc);
-	if (!filename || !strlen(filename) || filename[strlen(filename) - 1] == '/')
-	{
-		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-															_("Please enter a file name,\nnot a directory"));
-		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-		gtk_dialog_run (GTK_DIALOG (message));
-		gtk_widget_destroy (message);
+	if (!filename || !strlen (filename) || g_file_test (filename, G_FILE_TEST_IS_DIR)) {
+		GetCmdContext ()->Message (_("Please enter a file name,\nnot a directory"), CmdContext::SeverityWarning, true);
 		return true;
 	}
 	int file_type = -1;
 	int n = strlen (filename), i = n - 1;
 	char const *pixbuf_type = NULL;
 	string filename2 = filename;
-	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
-	if (filename[i] == '/') i = 0;
+	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/'))
+		i--;
+	if (filename[i] == '/')
+		i = 0;
 	ext = (i > 0)? filename + i + 1: NULL;
 	if (!mime_type) // to be really sure we don't crash
 		mime_type = "application/x-gchempaint";
@@ -568,11 +573,8 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 		char *unescaped = g_uri_unescape_string (filename, NULL);
 		char *mess = g_strdup_printf (_("Sorry, format %s not supported!\nFailed to load %s."), mime_type, unescaped);
 		g_free (unescaped);
-		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-													mess);
-		gtk_dialog_run (GTK_DIALOG (message));
+		GetCmdContext ()->Message (mess, CmdContext::SeverityError, true);
 		g_free (mess);
-		gtk_widget_destroy (message);
 		return true;
 	}
 	list<string> &exts = globs[mime_type];
@@ -594,31 +596,25 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 				filename2 += string(".") + default_ext;
 		file = g_file_new_for_uri (filename2.c_str ());
 		err = g_file_query_exists (file, NULL);
-		gint result = GTK_RESPONSE_YES;
+		CmdContext::Response result = CmdContext::ResponseYes;
 		if (err) {
 			char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
 			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), unescaped);
 			g_free (unescaped);
-			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
-			result = gtk_dialog_run (Box);
-			gtk_widget_destroy (GTK_WIDGET (Box));
+			result = GetCmdContext ()->GetResponse (message, CmdContext::ResponseYes | CmdContext::ResponseNo);
 			g_free (message);
 		}
-		if (result == GTK_RESPONSE_YES) {
+		if (result == CmdContext::ResponseYes) {
 			// destroy the old file if needed
 			if (err) {
 				GError *error = NULL;
 				g_file_delete (file, NULL, &error);
 				if (error) {
 					char *unescaped = g_uri_unescape_string (filename2.c_str (), NULL);
-					gchar * message = g_strdup_printf (_("Error while processing %s:\n%s"), unescaped, error->message);
+					gchar *message = g_strdup_printf (_("Error while processing %s:\n%s"), unescaped, error->message);
 					g_free (unescaped);
 					g_error_free (error);
-					GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-					gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
-					result = gtk_dialog_run (Box);
-					gtk_widget_destroy (GTK_WIDGET (Box));
+					GetCmdContext ()->Message (message, CmdContext::SeverityError, false);
 					g_free (message);
 					g_object_unref (file);
 					return false;
@@ -736,11 +732,8 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 		// Note to translator: add a space if needed before the semicolon
 		mess += _(":\n");
 		mess += e.what ();
-		GtkWidget *message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess.c_str (), NULL, NULL);
+		GetCmdContext ()->Message (mess.c_str (), CmdContext::SeverityError, false);
 		g_free (unescaped);
-		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
-		gtk_widget_show (message);
 	}
 	return false;
 }
@@ -762,7 +755,6 @@ void Application::SaveWithBabel (string const &filename, const gchar *mime_type,
 
 void Application::OpenWithBabel (string const &filename, const gchar *mime_type, Document* pDoc)
 {
-	string old_num_locale;
 	bool bNew = (pDoc == NULL || !pDoc->GetEmpty () || pDoc->GetDirty ());
 	GFile *file;
 	GFileInfo *info = NULL;
@@ -809,8 +801,6 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 		g_object_unref (file);
 		buf[size] = 0;
 		istringstream iss (buf);
-		old_num_locale = setlocale (LC_NUMERIC, NULL);
-		setlocale(LC_NUMERIC, "C");
 		OBMol Mol;
 		OBConversion Conv;
 		OBFormat* pInFormat = Conv.FormatFromExt (filename.c_str ());
@@ -825,7 +815,6 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 			if (!result)
 				break;
 		}
-		setlocale (LC_NUMERIC, old_num_locale.c_str ());
 		delete [] buf;
 		if (!result)
 		{
@@ -864,7 +853,6 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 	catch (int num)
 	{
 		gchar *mess = NULL;
-		GtkWidget* message;
 		switch (num)
 		{
 		case 0:
@@ -879,12 +867,9 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 		default:
 			throw (num); //this should not occur
 		}
-		char *unescaped = g_uri_unescape_string (filename.c_str (), NULL);
-		message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, unescaped, NULL);
+		char *unescaped = g_strdup_printf (mess, g_uri_unescape_string (filename.c_str (), NULL));
+		GetCmdContext ()->Message (unescaped, CmdContext::SeverityError, false);
 		g_free (unescaped);
-		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
-		gtk_widget_show(message);
 	}
 }
 
@@ -917,7 +902,6 @@ static int	cb_vfs_to_xml (GInputStream *input, char* buf, int nb)
 void Application::OpenGcp (string const &filename, Document* pDoc)
 {
 	xmlDocPtr xml = NULL;
-	char *old_num_locale, *old_time_locale;
 	GError *error = NULL;
 	GFileInfo *info = NULL;
 	bool create = false;
@@ -956,10 +940,6 @@ void Application::OpenGcp (string const &filename, Document* pDoc)
 			throw (int) 2;
 		if (strcmp((char*)xml->children->name, "chemistry"))
 			throw (int) 3;	//FIXME: that could change when a dtd is available
-		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
-		setlocale(LC_NUMERIC, "C");
-		old_time_locale = g_strdup(setlocale(LC_TIME, NULL));
-		setlocale(LC_TIME, "C");
 		if (!pDoc || !pDoc->GetEmpty () || pDoc->GetDirty ()) {
 			create = true;
 			OnFileNew ();
@@ -967,10 +947,6 @@ void Application::OpenGcp (string const &filename, Document* pDoc)
 		}
 		pDoc->SetFileName(filename, "application/x-gchempaint");
 		bool result = pDoc->Load(xml->children);
-		setlocale(LC_NUMERIC, old_num_locale);
-		g_free(old_num_locale);
-		setlocale(LC_TIME, old_time_locale);
-		g_free(old_time_locale);
 		if (!result) {
 			if (create)
 				pDoc->GetWindow ()->Destroy ();
