@@ -30,7 +30,6 @@
 #include "reaction-operator.h"
 #include "document.h"
 #include "mechanism-arrow.h"
-#include "molecule.h"
 #include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
@@ -134,9 +133,17 @@ xmlNodePtr ReactionStep::Save (xmlDocPtr xml) const
 	
 bool ReactionStep::Load (xmlNodePtr node)
 {
-	m_bLoading = true;
 	if (!Object::Load (node))
 		return false;
+	Document *pDoc = dynamic_cast <Document*> (GetDocument ());
+	pDoc->NotifyDirty (this);
+	pDoc->ObjectLoaded (this);
+	return true;
+}
+
+void ReactionStep::OnLoaded ()
+{
+	m_bLoading = true;
 	map<Object*, gccv::Rect> Objects;
 	map<double, Object*> Children;
 	gccv::Rect rect;
@@ -145,6 +152,7 @@ bool ReactionStep::Load (xmlNodePtr node)
 	Document *pDoc = dynamic_cast <Document*> (GetDocument ());
 	View *view = pDoc->GetView ();
 	Theme *pTheme = pDoc->GetTheme ();
+	view->Update (this);
 	WidgetData  *pData= view->GetData ();
 	map<double, Object*>::iterator im, endm;
 	double x, y, x0, y0, x1, y1;
@@ -178,18 +186,16 @@ bool ReactionStep::Load (xmlNodePtr node)
 		rect = Objects[pObj];
 		x+= rect.x1 - rect.x0;
 	}
-	view->Update (this);
 	m_bLoading = false;
-	return true;
 }
 
 double ReactionStep::GetYAlign ()
 {
 	map<string, Object*>::iterator i;
-	Object *obj = GetFirstChild (i);
-	while (obj && obj->GetType () != gcu::ReactantType && obj->GetType () != MechanismStepType)
-		obj = GetNextChild (i);
-	return (obj)? obj->GetYAlign (): 0.;
+	GetFirstChild (i);
+	while ((*i).second->GetType () != gcu::ReactantType && (*i).second->GetType () != MechanismStepType)
+		GetNextChild (i);
+	return ((*i).second)? (*i).second->GetYAlign (): 0.;
 }
 
 bool ReactionStep::OnSignal (SignalId Signal, G_GNUC_UNUSED Object *Child)
@@ -344,11 +350,4 @@ void ReactionStep::CleanChildren ()
 	}
 }
 
-void ReactionStep::AddMolecule (Molecule *molecule, bool signal)
-{
-	new Reactant (this, molecule);
-	if (signal)
-		EmitSignal (OnChangedSignal);
-}
-
 }	//	namespace gcp
