@@ -22,6 +22,7 @@
 
 #include "config.h"
 #include "application.h"
+#include "objprops.h"
 #include "spectrumdoc.h"
 #include "spectrumview.h"
 #include <glib/gi18n-lib.h>
@@ -29,6 +30,10 @@
 #include <sstream>
 #include <iostream>
 
+#ifndef HAVE_EXP10
+#	define exp10(x) pow(10.,(x))
+#endif
+
 using namespace std;
 
 namespace gcu
@@ -62,7 +67,7 @@ SpectrumDocument::SpectrumDocument (Application *App, SpectrumView *View):
 {
 	m_View = (View)? View: new SpectrumView (this);
 	x = y = NULL;
-	X = Y = R = I = integral = Rt = It = Rp = -1;
+	X = Xt = Y = R = I = integral = Rt = It = Rp = -1;
 	npoints = 0;
 	maxx = maxy = minx = miny = go_nan;
 	firstx = lastx = deltax = firsty = go_nan;
@@ -613,23 +618,26 @@ static void on_show_integral (GtkButton *btn, SpectrumDocument *doc)
 	doc->OnShowIntegral ();
 }
 
-/*
+
 static void on_transform_fid (GtkButton *btn, SpectrumDocument *doc)
 {
 	doc->OnTransformFID (btn);
 }
-*/
 
 static SpectrumType get_spectrum_type_from_string (char const *buf)
 {
 	unsigned i;
+	char *up = g_ascii_strup (buf, -1);
 	for (i = 0; i < G_N_ELEMENTS (Types); i++)
-		if (!strcmp (Types[i].name, buf))
-		return Types[i].type;
+		if (!strcmp (Types[i].name, up)) {
+			g_free (up);
+			return Types[i].type;
+		}
+	g_free (up);
 	return GCU_SPECTRUM_MAX;
 }
 
-#define JCAMP_PREC 1e-3 // fully arbitrary
+#define JCAMP_PREC 1e-2 // fully arbitrary
 
 void SpectrumDocument::LoadJcampDx (char const *data)
 {
@@ -720,10 +728,10 @@ parse_line:
 							cur++;
 						if (*cur == 0)
 							break;
-						x[read] = strtod (cur, &cur);
+						x[read] = g_ascii_strtod (cur, &cur);
 						while (*cur && (*cur < '0' || *cur > '9') && *cur != '-' && *cur !='+')
 							cur++;
-						y[read] = strtod (cur, &cur);
+						y[read] = g_ascii_strtod (cur, &cur);
 						read++;
 					}
 				}
@@ -745,16 +753,16 @@ parse_line:
 		case JCAMP_YLABEL:
 			break;
 		case JCAMP_XFACTOR:
-			xfactor = strtod (buf, NULL);
+			xfactor = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_YFACTOR:
-			yfactor = strtod (buf, NULL);
+			yfactor = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_FIRSTX:
-			firstx = strtod (buf, NULL);
+			firstx = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_LASTX:
-			lastx = strtod (buf, NULL);
+			lastx = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_NPOINTS: {
 			unsigned n = (unsigned) atoi (buf);
@@ -770,19 +778,19 @@ parse_line:
 			break;
 		}
 		case JCAMP_FIRSTY:
-			firsty = strtod (buf, NULL);
+			firsty = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_MAXX:
-			maxx = strtod (buf, NULL);
+			maxx = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_MINX:
-			minx = strtod (buf, NULL);
+			minx = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_MAXY:
-			maxy = strtod (buf, NULL);
+			maxy = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_MINY:
-			miny = strtod (buf, NULL);
+			miny = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_RUNITS:
 		case JCAMP_AUNITS:
@@ -959,7 +967,7 @@ parse_line:
 			char *cur = buf;
 			double x;
 			while (*cur) {
-				x = strtod (cur, &cur);
+				x = g_ascii_strtod (cur, &cur);
 				if (*cur)
 					cur++;
 				if (i < variables.size ())
@@ -978,7 +986,7 @@ parse_line:
 			char *cur = buf;
 			double x;
 			while (*cur) {
-				x = strtod (cur, &cur);
+				x = g_ascii_strtod (cur, &cur);
 				if (*cur)
 					cur++;
 				if (i < variables.size ())
@@ -997,7 +1005,7 @@ parse_line:
 			char *cur = buf;
 			double x;
 			while (*cur) {
-				x = strtod (cur, &cur);
+				x = g_ascii_strtod (cur, &cur);
 				if (*cur)
 					cur++;
 				if (i < variables.size ())
@@ -1016,7 +1024,7 @@ parse_line:
 			char *cur = buf;
 			double x;
 			while (*cur) {
-				x = strtod (cur, &cur);
+				x = g_ascii_strtod (cur, &cur);
 				if (*cur)
 					cur++;
 				if (i < variables.size ())
@@ -1035,7 +1043,7 @@ parse_line:
 			char *cur = buf;
 			double x;
 			while (*cur) {
-				x = strtod (cur, &cur);
+				x = g_ascii_strtod (cur, &cur);
 				if (*cur)
 					cur++;
 				if (i < variables.size ())
@@ -1143,7 +1151,7 @@ parse_line:
 			break;
 		}
 		case JCAMP_DELTAX:
-			deltax = strtod (buf, NULL);
+			deltax = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_CLASS:
 		case JCAMP_ORIGIN:
@@ -1193,7 +1201,7 @@ parse_line:
 		case JCAMP_MONOISOTOPIC_MASS:
 			break;
 		case JCAMP_OBSERVE_FREQUENCY:
-			freq = strtod (buf, NULL);
+			freq = g_ascii_strtod (buf, NULL);
 			break;
 		case JCAMP_OBSERVE_NUCLEUS:
 		case JCAMP_SOLVENT_REFERENCE:
@@ -1216,10 +1224,10 @@ parse_line:
 		case JCAMP_RELAXATION_TIMES:
 			break;
 		case BRUCKER_OFFSET:
-			offset = strtod (buf, NULL);
+			offset = g_ascii_strtod (buf, NULL);
 			break;
 		case VARIAN_OFFSET:
-			refpoint = strtod (buf, NULL);
+			refpoint = g_ascii_strtod (buf, NULL);
 			break;
 		default:
 			break;
@@ -1227,186 +1235,7 @@ parse_line:
 	}
 
 out:
-	bool hide_y_axis = false;
-	SpectrumUnitType unit = GCU_SPECTRUM_UNIT_MAX;
-	// doon't do anything for unsupported spectra
-	switch (m_SpectrumType) {
-	case GCU_SPECTRUM_NMR: {
-		if (x == NULL && X > 0 && variables[X].Values == NULL)
-			return;
-		// fix origin
-		if (go_finite (offset)) {
-			unsigned i;
-			if (x) {
-				double d = offset * freq - maxx;
-				maxx += d;
-				minx += d;
-				firstx += d;
-				lastx += d;
-				for (i = 0; i < npoints; i++)
-					x[i] += d;
-			} else {
-				double d = offset * freq - variables[X].Max;
-				maxx = variables[X].Max += d;
-				minx = variables[X].Min += d;
-				variables[X].First += d;
-				variables[X].Last += d;
-				for (i = 0; i < npoints; i++)
-					variables[X].Values[i] += d;
-			}
-		} else if (go_finite (refpoint)) {
-			unsigned i;
-			double d = -refpoint;
-			if (x) {
-				maxx += d;
-				minx += d;
-				firstx += d;
-				lastx += d;
-				for (i = 0; i < npoints; i++)
-					x[i] += d;
-			} else {
-				maxx = variables[X].Max += d;
-				minx = variables[X].Min += d;
-				variables[X].First += d;
-				variables[X].Last += d;
-				for (i = 0; i < npoints; i++)
-					variables[X].Values[i] += d;
-			}
-		}
-		// add some widgets to the option box
-		GtkWidget *box = gtk_hbox_new (false, 5), *w;
-		if (go_finite (freq)) {
-			w = gtk_label_new (_("X unit:"));
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			w = gtk_combo_box_new_text ();
-			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Chemical shift (ppm)"));
-			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Frequency (Hz)"));
-			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
-			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_PPM)? 0: 1));
-			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-		}
-		w = gtk_button_new_with_label (_("Show integral"));
-		g_signal_connect (w, "clicked", G_CALLBACK (on_show_integral), this);
-		gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-		gtk_widget_show_all (box);
-		gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
-		hide_y_axis = true;
-		break;
-	}
-	case GCU_SPECTRUM_NMR_FID: {
-		if (x == NULL && X > 0 && variables[X].Values == NULL)
-			return;
-// Deactivate Fourier transfor for now
-/*		if (R >= 0 && I >= 0) {
-			GtkWidget *box = gtk_hbox_new (false, 5);
-			GtkWidget *w = gtk_button_new_with_label (_("Transform to spectrum"));
-			g_signal_connect (w, "clicked", G_CALLBACK (on_transform_fid), this);
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			gtk_widget_show_all (box);
-			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
-		}
-		hide_y_axis = true;*/
-		break;
-	}
-	case GCU_SPECTRUM_INFRARED:
-	case GCU_SPECTRUM_RAMAN:
-		unit = GCU_SPECTRUM_UNIT_MICROMETERS;
-//	case GCU_SPECTRUM_INFRARED_PEAK_TABLE:
-//	case GCU_SPECTRUM_INFRARED_INTERFEROGRAM:
-//	case GCU_SPECTRUM_INFRARED_TRANSFORMED:
-	case GCU_SPECTRUM_UV_VISIBLE:
-		if (x == NULL && X > 0 && variables[X].Values == NULL)
-			return;
-		else {
-			if (unit == GCU_SPECTRUM_UNIT_MAX)
-				unit = GCU_SPECTRUM_UNIT_NANOMETERS;
-			// add some widgets to the option box
-			GtkWidget *box = gtk_hbox_new (false, 5), *w;
-			w = gtk_label_new (_("X unit:"));
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			w = gtk_combo_box_new_text ();
-			if  (unit == GCU_SPECTRUM_UNIT_NANOMETERS)
-				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (nm)"));
-			else
-				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (µm)"));
-			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave number (1/cm)"));
-			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
-			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_CM_1)? 1: 0));
-			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			m_XAxisInvertBtn = gtk_check_button_new_with_label (_("Invert X Axis"));
-			m_XAxisInvertSgn = g_signal_connect (m_XAxisInvertBtn, "toggled", G_CALLBACK (on_xaxis_invert), this);
-			gtk_box_pack_start (GTK_BOX (box), m_XAxisInvertBtn, false, false, 0);
-			w = gtk_vseparator_new ();
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 12);
-			w = gtk_label_new (_("Y unit:"));
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			w = gtk_combo_box_new_text ();
-			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Absorbance"));
-			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Transmittance"));
-			unit = (Y >= 0)? variables[Y].Unit: m_YUnit;
-			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_ABSORBANCE)? 0: 1));
-			g_signal_connect (w, "changed", G_CALLBACK (on_yunit_changed), this);
-			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
-			gtk_widget_show_all (box);
-			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
-		}
-		break;
-//	case GCU_SPECTRUM_NMR_PEAK_TABLE:
-//	case GCU_SPECTRUM_NMR_PEAK_ASSIGNMENTS:
-	case GCU_SPECTRUM_MASS:
-		break;
-	default:
-		return;
-	}
-	m_Empty = npoints == 0;
-	GOData *godata;
-	GogSeries *series = m_View->GetSeries ();
-	if (X >= 0 && variables[X].Values != NULL) {
-		godata = go_data_vector_val_new (variables[X].Values, npoints, NULL);
-		m_XUnit = variables[X].Unit;
-	}
-	else
-		godata = go_data_vector_val_new (x, npoints, NULL);
-	gog_series_set_dim (series, 0, godata, NULL);
-	if (Y >= 0 && variables[Y].Values != NULL) {
-		godata = go_data_vector_val_new (variables[Y].Values, npoints, NULL);
-		m_YUnit = variables[Y].Unit;
-	} else if (R >= 0 && variables[R].Values != NULL) {
-		godata = go_data_vector_val_new (variables[R].Values, npoints, NULL);
-		m_YUnit = variables[R].Unit;
-	} else
-		godata = go_data_vector_val_new (y, npoints, NULL);
-	gog_series_set_dim (series, 1, godata, NULL);
-	/* invert X-axis if needed */
-	bool invert_axis = false;
-	switch (m_XUnit) {
-	case GCU_SPECTRUM_UNIT_CM_1:
-	case GCU_SPECTRUM_UNIT_PPM:
-		invert_axis = true;
-		break;
-	case GCU_SPECTRUM_UNIT_HZ:
-		if (m_SpectrumType == GCU_SPECTRUM_NMR)
-			invert_axis = true;
-		break;
-	default:
-		break;
-	}
-	if (m_XAxisInvertBtn) {
-		g_signal_handler_block (m_XAxisInvertBtn, m_XAxisInvertSgn);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_XAxisInvertBtn), invert_axis);
-		g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
-	}
-	m_View->SetAxisBounds (GOG_AXIS_X, minx, maxx, invert_axis);
-	m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, false);
-	if (hide_y_axis)
-		m_View->ShowAxis (GOG_AXIS_Y, false);
-	/* Add axes labels */
-	if (m_XUnit < GCU_SPECTRUM_UNIT_MAX)
-		m_View->SetAxisLabel (GOG_AXIS_X, _(UnitNames[m_XUnit]));
-	if (m_YUnit < GCU_SPECTRUM_UNIT_MAX)
-		m_View->SetAxisLabel (GOG_AXIS_Y, _(UnitNames[m_YUnit]));
+	Loaded ();
 }
 
 void SpectrumDocument::ReadDataLine (char const *data, list<double> &l)
@@ -1548,7 +1377,7 @@ void SpectrumDocument::ReadDataLine (char const *data, list<double> &l)
 			buf[j++] = c;
 		}
 		buf[j] = 0;
-		newval = strtod (buf, NULL);
+		newval = g_ascii_strtod (buf, NULL);
 		if (!pos)
 			newval = - newval;
 		if (diff)
@@ -1992,10 +1821,12 @@ void SpectrumDocument::OnShowIntegral ()
 			v.Values = new double[v.NbValues];
 			unsigned i;
 			double *z;
-			if (R >= 0)
+			if (Rp >= 0)
+				z = variables[Rp].Values;
+			else if (R >= 0)
 				z = variables[R].Values;
 			else if (Y >= 0)
-				z = variables[R].Values;
+				z = variables[Y].Values;
 			else
 				z = y;
 			xo = (X >= 0 && variables[X].Values != NULL)? variables[X].Values: x;
@@ -2028,6 +1859,9 @@ void SpectrumDocument::OnShowIntegral ()
 				v.Values[i] -= res[4] * (acc *= cur);
 				v.Values[i] -= res[5] * cur * acc;
 			}
+			if (xo[1] > xo[0])
+				for (i = 0; i < v.NbValues; i++)
+					v.Values[i] = -v.Values[i];
 			g_free (reg.se);
 			g_free (reg.t);
 			g_free (reg.xbar);
@@ -2077,7 +1911,6 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	unsigned n = 2;
 	while (n < npoints)
 		n <<= 1;
-	n <<= 1; // doubles the points number to get more transformed values
 	go_complex *fid = new go_complex[n], *sp;
 	unsigned i;
 	for (i = 0; i < npoints; i++) {
@@ -2092,10 +1925,8 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	//we make no apodization at the moment
 	go_fourier_fft (fid, n, 1, &sp, false);
 	delete [] fid;
-	// only the first half values are useful
-	n /= 2;
 	// copy the unphased data to Rt and It (t for transformed)
-	JdxVar vr, vi, rp;
+	JdxVar vr, vi, rp, xt;
 	vr.Name = _("Real transformed data");
 	vr.Symbol = 't';
 	vr.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
@@ -2104,15 +1935,6 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	vr.Factor = 1.;
 	vr.NbValues = n;
 	vr.Values = new double[n];
-	for (i = 0; i < n; i++)
-		vr.Values[i] = sp[i].re;
-	vr.First = vr.Values[0];
-	vr.Last = vr.Values[n - 1];
-	go_range_min (vr.Values, n, &vr.Min);
-	go_range_max (vr.Values, n, &vr.Max);
-	vr.Series = NULL;
-	Rt = variables.size ();
-	variables.push_back (vr);
 	vi.Name = _("Imaginary transformed data");
 	vi.Symbol = 'u';
 	vi.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
@@ -2121,8 +1943,25 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	vi.Factor = 1.;
 	vi.NbValues = n;
 	vi.Values = new double[n];
-	for (i = 0; i < n; i++)
-		vi.Values[i] = sp[i].im;
+	unsigned n2 = n / 2 - 1, j;
+	for (i = 0, j = n2; i < n2; i++, j--) {
+		vr.Values[i] = sp[j].re;
+		vi.Values[i] = sp[j].im;
+	}
+	// the value at 0 must be skipped, doing a linear interpolation
+	vr.Values[i] = (sp[1].re + sp[n - 1].re) / 2.;
+	vi.Values[i++] = (sp[1].im + sp[n - 1].im) / 2.;
+	for (j = n - 1; i < n; i++, j--) {
+		vr.Values[i] = sp[j].re;
+		vi.Values[i] = sp[j].im;
+	}
+	vr.First = vr.Values[0];
+	vr.Last = vr.Values[n - 1];
+	go_range_min (vr.Values, n, &vr.Min);
+	go_range_max (vr.Values, n, &vr.Max);
+	vr.Series = NULL;
+	Rt = variables.size ();
+	variables.push_back (vr);
 	vi.First = vi.Values[0];
 	vi.Last = vi.Values[n - 1];
 	go_range_min (vi.Values, n, &vi.Min);
@@ -2131,11 +1970,157 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	It = variables.size ();
 	variables.push_back (vi);
 	// Now we need to adjust the phase (see http://www.ebyte.it/stan/Poster_EDISPA.html)
-	double phi = 0., tau = 0., *z;
-	double step = M_PI * 2. * tau / n, phase;
+	double phi = 0., tau = 0., *z, maxz = 0., phiopt = 0., tauopt = 0.;
 	z = new double[n];
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+		// copy reordered data to sp
+		sp[i].re = vr.Values[i];
+		sp[i].im = vi.Values[i];
 		z[i] = go_complex_mod (sp + i);
+		if (z[i] > maxz)
+			maxz = z[i];
+	}
+	// normalize the z values
+	for (i = 0; i < n; i++)
+		z[i] /= maxz;
+	double c = 0.1;
+	unsigned nmax, nmin;
+	while (c > 0.) {
+		nmin = n, nmax = 0;
+		for (i = 0; i < n; i++)
+			if (z[i] > c) {
+				nmin = i;
+				break;
+			}
+		for (i = n - 1; i > 0; i--)
+			if (z[i] > c) {
+				nmax = i;
+				break;
+			}
+		if ((nmax - nmin) > n / 10) // 10 is arbitrary, but should be not too large
+			break;
+		c /= 2.;
+	}
+	// make a list of indices with z greater than c
+	std::list <unsigned> restricted;
+	for (i = 0; i < n; i++)
+		if (z[i] > c)
+			restricted.push_back (i);
+	// evaluate the predictor in c, since the value is not used anymore
+	double maxc = 0., maxk = 0., p, phis[41], cs[41];
+	unsigned k, kopt = 0;
+	std::list <unsigned>::iterator it, itend = restricted.end ();
+	for (k = 0; k < 41; k++) {
+		tau = -1. + k * .1;
+		maxk = 0.;
+		for (phi = 0; phi <  2 * M_PI; phi += 10. / 180. * M_PI) {
+			c = 0.;
+			for (it = restricted.begin (); it != itend; it++) {
+				i = *it;
+				p = phi - 2. * M_PI * tau * (n - 1 - i) / n;
+				c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+			}
+			if (c > maxk) {
+				if (c > maxc) {
+					maxc = c;
+					tauopt = tau;
+					phiopt = phi;
+					kopt = k;
+				}
+				maxk = c;
+				phis[k] = phi;
+			}
+		}
+		cs[k] = maxk;
+	}
+	// let's search more finely around the maximum
+	// using the fact that phis[k] is a(n almost) linear function of tau
+	// first, reevaluate the optimum phi for tauopt and previous value and next value with a step of 1°
+	double phase = phiopt;
+	for (phi = phase - 9. / 180. * M_PI; phi < phase + 10. / 180. * M_PI; phi += 1. / 180. * M_PI) {
+			c = 0.;
+			for (it = restricted.begin (); it != itend; it++) {
+				i = *it;
+				p = phi - 2. * M_PI * tauopt * (n - 1 - i) / n;
+				c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+			}
+			if (c > maxc) {
+				maxc = c;
+				phiopt = phi;
+			}
+	}
+	for (k = 1; k < 10; k++) {
+		// search with lower values of tau
+		tau = tauopt - 0.01 * k;
+		for (phi = 0; phi <  2 * M_PI; phi += 10. / 180. * M_PI) {
+			c = 0.;
+			maxk = 0;
+			for (it = restricted.begin (); it != itend; it++) {
+				i = *it;
+				p = phi - 2. * M_PI * tau * (n - 1 - i) / n;
+				c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+			}
+			if (c > maxk) {
+				phis[0] = phi;
+				maxk = c;
+			}
+		}
+		phase = phis[0];
+		for (phi = phase - 9. / 180. * M_PI; phi < phase + 10. / 180. * M_PI; phi += 1. / 180. * M_PI) {
+				c = 0.;
+				for (it = restricted.begin (); it != itend; it++) {
+					i = *it;
+					p = phi - 2. * M_PI * tau * (n - 1 - i) / n;
+					c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+				}
+				if (c > maxk) {
+					maxk = c;
+					phis[0] = phi;
+				}
+		}
+		if (maxk < maxc)
+			break;
+		maxc = maxk;
+		phiopt = phis[0];
+		tauopt = tau;
+	}
+	if (k == 1) {
+		for (k = 1; k < 10; k++) {
+			// search with higher values of tau
+			tau = tauopt + 0.01 * k;
+			for (phi = 0; phi <  2 * M_PI; phi += 10. / 180. * M_PI) {
+				c = 0.;
+				maxk = 0;
+				for (it = restricted.begin (); it != itend; it++) {
+					i = *it;
+					p = phi - 2. * M_PI * tau * (n - 1 - i) / n;
+					c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+				}
+				if (c > maxk) {
+					phis[0] = phi;
+					maxk = c;
+				}
+			}
+			phase = phis[0];
+			for (phi = phase - 9. / 180. * M_PI; phi < phase + 10. / 180. * M_PI; phi += 1. / 180. * M_PI) {
+					c = 0.;
+					for (it = restricted.begin (); it != itend; it++) {
+						i = *it;
+						p = phi - 2. * M_PI * tau * (n - 1 - i) / n;
+						c += z[i] * z[i] * (sp[i].re * cos (p) - sp[i].im * sin (p)) * exp (-fabs (4. * i / n -2));
+					}
+					if (c > maxk) {
+						maxk = c;
+						phis[0] = phi;
+					}
+			}
+			if (maxk < maxc)
+				break;
+			maxc = maxk;
+			phiopt = phis[0];
+			tauopt = tau;
+		}
+	}
 	g_free (sp);
 	// set the phase real values
 	// free what needs to be freed
@@ -2143,33 +2128,85 @@ void SpectrumDocument::OnTransformFID (G_GNUC_UNUSED GtkButton *btn)
 	// set the corrected values
 	rp.Values = new double[n];
 	//store phi and tau as first and last, respectively
-	rp.First = phi;
-	rp.Last = tau;
-	phase = phi - M_PI * 2. * tau;
+	double step = M_PI * 2. * tauopt / n;
+	phase = phiopt - M_PI * 2. * tauopt;
+	rp.First = phase + step;
+	rp.Last = phiopt + n * step;
 	for (i = 0; i < n; i++) {
 		phase += step;
-		rp.Values[i] = vr.Values[i] * cos (phase) + vi.Values[i] * sin (phase);
+		rp.Values[i] = vr.Values[i] * cos (phase) - vi.Values[i] * sin (phase);
 	}
+	go_range_min (rp.Values, n, &rp.Min);
+	go_range_max (rp.Values, n, &rp.Max);
 	Rp = variables.size ();
 	variables.push_back (rp);
 	// add Hz and ppm variables (0 for last point, user will have to choose a reference peak)
 	// first Hz
 	// if we are there, we have R and I values, we should have also X, but let's check
-	double *xo, freq;
+	double *xo, freq, shift;
 	if (X >= 0 && variables[X].Values != NULL) {
 		xo = variables[X].Values;
-		freq = (variables[X].NbValues - 1) / (variables[X].Last - variables[X].First);
+		freq = 1 / (variables[X].Last - variables[X].First);
 	} else {
 		xo = x;
-		freq = (npoints - 1) / (lastx - firstx);
+		freq = 1 / (lastx - firstx);
 	}
+	if (!go_finite (offset))
+		offset = 0.;
+	shift = offset - freq * (npoints - 1) / 2.;
 	//now display the spectrum
 	variables[R].Series = NULL;
 	rp.Series = m_View->GetSeries ();
 	GOData *godata = go_data_vector_val_new (rp.Values, n, NULL);
 	gog_series_set_dim (rp.Series, 1, godata, NULL);
-	godata = (X < 0)? go_data_vector_val_new (x, npoints, NULL): go_data_vector_val_new (variables[X].Values, variables[X].NbValues, NULL);
+	m_View->SetAxisBounds (GOG_AXIS_Y, rp.Min, rp.Max, false);
+	if (Xt < 0) {
+		xt.Name = _("Chemical shift");
+		xt.Symbol = 'X';
+		xt.Type = GCU_SPECTRUM_TYPE_INDEPENDENT;
+		xt.Unit = GCU_SPECTRUM_UNIT_HZ;
+		xt.Format = GCU_SPECTRUM_FORMAT_MAX;
+		xt.Factor = 1.;
+		xt.NbValues = n;
+		xt.Values = new double[n];
+		for (i = 0; i < n; i++)
+			xt.Values[i] = i * freq + shift;
+		xt.Min = xt.First = xt.Values[0];
+		xt.Max = xt.Last = xt.Values[n - 1];
+		xt.Series = NULL;
+		X = variables.size ();
+		variables.push_back (xt);
+	}
+	godata = go_data_vector_val_new (variables[X].Values, variables[X].NbValues, NULL);
 	gog_series_set_dim (rp.Series, 0, godata, NULL);
+	m_SpectrumType = GCU_SPECTRUM_NMR;
+	m_View->SetAxisBounds (GOG_AXIS_X, variables[X].Min, variables[X].Max, true);
+	m_View->SetAxisLabel (GOG_AXIS_X, _(UnitNames[variables[X].Unit]));
+	OnXUnitChanged (0);
+	// remove the last widget from the option box, really a kludge
+	GtkContainer *container = GTK_CONTAINER (m_View->GetOptionBox ());
+	GList *l = gtk_container_get_children (container), *ptr;
+	for (ptr = l; ptr->next != NULL; ptr = ptr->next);
+	gtk_container_remove (container, GTK_WIDGET (ptr->data));
+	g_list_free (l);
+	// now add the widgets appropriate for an NMR spectrum
+	GtkWidget *box = gtk_hbox_new (false, 5), *w;
+	if (go_finite (freq)) {
+		w = gtk_label_new (_("X unit:"));
+		gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+		w = gtk_combo_box_new_text ();
+		gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Chemical shift (ppm)"));
+		gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Frequency (Hz)"));
+		SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
+		gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_PPM)? 0: 1));
+		g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
+		gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+	}
+	w = gtk_button_new_with_label (_("Show integral"));
+	g_signal_connect (w, "clicked", G_CALLBACK (on_show_integral), this);
+	gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+	gtk_widget_show_all (box);
+	gtk_box_pack_start (GTK_BOX (container), box, false, false, 0);
 }
 
 void SpectrumDocument::OnXAxisInvert (bool inverted)
@@ -2177,4 +2214,305 @@ void SpectrumDocument::OnXAxisInvert (bool inverted)
 	m_View->InvertAxis (GOG_AXIS_X, inverted);
 }
 
+bool SpectrumDocument::Loaded () throw (gcu::LoaderError)
+{
+	bool hide_y_axis = false;
+	SpectrumUnitType unit = GCU_SPECTRUM_UNIT_MAX;
+	// doon't do anything for unsupported spectra
+	switch (m_SpectrumType) {
+	case GCU_SPECTRUM_NMR: {
+		if (x == NULL && X >= 0 && variables[X].Values == NULL)
+			return false;
+		// fix origin
+		if (go_finite (offset)) {
+			unsigned i;
+			if (x) {
+				double d = offset * freq - maxx;
+				maxx += d;
+				minx += d;
+				firstx += d;
+				lastx += d;
+				for (i = 0; i < npoints; i++)
+					x[i] += d;
+			} else if (X < 0) {
+				JdxVar xt;
+				xt.Name = _("Chemical shift");
+				xt.Symbol = 'X';
+				xt.Type = GCU_SPECTRUM_TYPE_INDEPENDENT;
+				xt.Unit = GCU_SPECTRUM_UNIT_HZ;
+				xt.Format = GCU_SPECTRUM_FORMAT_MAX;
+				xt.Factor = 1.;
+				xt.NbValues = npoints;
+				xt.Values = new double[npoints];
+				double freq = (maxx - minx) / npoints, shift = (maxx - minx) / 2. - offset;
+				maxx -= shift;
+				minx -= shift;
+				for (i = 0; i < npoints; i++)
+					xt.Values[i] = i * freq + minx;
+				xt.Min = xt.First = xt.Values[0];
+				xt.Max = xt.Last = xt.Values[npoints - 1];
+				xt.Series = NULL;
+				X = variables.size ();
+				variables.push_back (xt);
+				OnXUnitChanged (0);
+				minx = variables[X].Min;
+				maxx = variables[X].Max;
+			} else {
+				double d = offset * freq - variables[X].Max;
+				maxx = variables[X].Max += d;
+				minx = variables[X].Min += d;
+				variables[X].First += d;
+				variables[X].Last += d;
+				for (i = 0; i < npoints; i++)
+					variables[X].Values[i] += d;
+			}
+		} else if (go_finite (refpoint)) {
+			unsigned i;
+			double d = -refpoint;
+			if (x) {
+				maxx += d;
+				minx += d;
+				firstx += d;
+				lastx += d;
+				for (i = 0; i < npoints; i++)
+					x[i] += d;
+			} else {
+				maxx = variables[X].Max += d;
+				minx = variables[X].Min += d;
+				variables[X].First += d;
+				variables[X].Last += d;
+				for (i = 0; i < npoints; i++)
+					variables[X].Values[i] += d;
+			}
+		}
+		// add some widgets to the option box
+		GtkWidget *box = gtk_hbox_new (false, 5), *w;
+		if (go_finite (freq)) {
+			w = gtk_label_new (_("X unit:"));
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			w = gtk_combo_box_new_text ();
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Chemical shift (ppm)"));
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Frequency (Hz)"));
+			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_PPM)? 0: 1));
+			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+		}
+		w = gtk_button_new_with_label (_("Show integral"));
+		g_signal_connect (w, "clicked", G_CALLBACK (on_show_integral), this);
+		gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+		gtk_widget_show_all (box);
+		gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
+		hide_y_axis = true;
+		break;
+	}
+	case GCU_SPECTRUM_NMR_FID: {
+		if (x == NULL && X >= 0 && variables[X].Values == NULL)
+			return false;
+		else if (x == NULL && X < 0) {
+			x = new double[npoints];
+			deltax = (maxx - minx) / (npoints - 1);
+			for (unsigned i = 0; i < npoints;i++)
+				x[i] = minx + i * deltax;
+			firstx = 0;
+			lastx = x[npoints - 1];
+		}
+		if (R >= 0 && I >= 0) {
+			GtkWidget *box = gtk_hbox_new (false, 5);
+			GtkWidget *w = gtk_button_new_with_label (_("Transform to spectrum"));
+			g_signal_connect (w, "clicked", G_CALLBACK (on_transform_fid), this);
+			if (!go_finite (offset))
+				gtk_widget_set_sensitive (w, false);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			gtk_widget_show_all (box);
+			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
+		}
+		hide_y_axis = true;
+		break;
+	}
+	case GCU_SPECTRUM_INFRARED:
+	case GCU_SPECTRUM_RAMAN:
+		unit = GCU_SPECTRUM_UNIT_MICROMETERS;
+//	case GCU_SPECTRUM_INFRARED_PEAK_TABLE:
+//	case GCU_SPECTRUM_INFRARED_INTERFEROGRAM:
+//	case GCU_SPECTRUM_INFRARED_TRANSFORMED:
+	case GCU_SPECTRUM_UV_VISIBLE:
+		if (x == NULL && X > 0 && variables[X].Values == NULL)
+			return false;
+		else {
+			if (unit == GCU_SPECTRUM_UNIT_MAX)
+				unit = GCU_SPECTRUM_UNIT_NANOMETERS;
+			// add some widgets to the option box
+			GtkWidget *box = gtk_hbox_new (false, 5), *w;
+			w = gtk_label_new (_("X unit:"));
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			w = gtk_combo_box_new_text ();
+			if  (unit == GCU_SPECTRUM_UNIT_NANOMETERS)
+				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (nm)"));
+			else
+				gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave length (µm)"));
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Wave number (1/cm)"));
+			SpectrumUnitType unit = (X >= 0)? variables[X].Unit: m_XUnit;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_CM_1)? 1: 0));
+			g_signal_connect (w, "changed", G_CALLBACK (on_xunit_changed), this);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			m_XAxisInvertBtn = gtk_check_button_new_with_label (_("Invert X Axis"));
+			m_XAxisInvertSgn = g_signal_connect (m_XAxisInvertBtn, "toggled", G_CALLBACK (on_xaxis_invert), this);
+			gtk_box_pack_start (GTK_BOX (box), m_XAxisInvertBtn, false, false, 0);
+			w = gtk_vseparator_new ();
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 12);
+			w = gtk_label_new (_("Y unit:"));
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			w = gtk_combo_box_new_text ();
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Absorbance"));
+			gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Transmittance"));
+			unit = (Y >= 0)? variables[Y].Unit: m_YUnit;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (w), ((unit == GCU_SPECTRUM_UNIT_ABSORBANCE)? 0: 1));
+			g_signal_connect (w, "changed", G_CALLBACK (on_yunit_changed), this);
+			gtk_box_pack_start (GTK_BOX (box), w, false, false, 0);
+			gtk_widget_show_all (box);
+			gtk_box_pack_start (GTK_BOX (m_View->GetOptionBox ()), box, false, false, 0);
+		}
+		break;
+//	case GCU_SPECTRUM_NMR_PEAK_TABLE:
+//	case GCU_SPECTRUM_NMR_PEAK_ASSIGNMENTS:
+	case GCU_SPECTRUM_MASS:
+		break;
+	default:
+		return false;
+	}
+	m_Empty = npoints == 0;
+	GOData *godata;
+	GogSeries *series = m_View->GetSeries ();
+	if (X >= 0 && variables[X].Values != NULL) {
+		godata = go_data_vector_val_new (variables[X].Values, npoints, NULL);
+		m_XUnit = variables[X].Unit;
+	}
+	else
+		godata = go_data_vector_val_new (x, npoints, NULL);
+	gog_series_set_dim (series, 0, godata, NULL);
+	if (Y >= 0 && variables[Y].Values != NULL) {
+		godata = go_data_vector_val_new (variables[Y].Values, npoints, NULL);
+		m_YUnit = variables[Y].Unit;
+	} else if (R >= 0 && variables[R].Values != NULL) {
+		godata = go_data_vector_val_new (variables[R].Values, npoints, NULL);
+		m_YUnit = variables[R].Unit;
+	} else
+		godata = go_data_vector_val_new (y, npoints, NULL);
+	gog_series_set_dim (series, 1, godata, NULL);
+	/* invert X-axis if needed */
+	bool invert_axis = false;
+	switch (m_XUnit) {
+	case GCU_SPECTRUM_UNIT_CM_1:
+	case GCU_SPECTRUM_UNIT_PPM:
+		invert_axis = true;
+		break;
+	case GCU_SPECTRUM_UNIT_HZ:
+		if (m_SpectrumType == GCU_SPECTRUM_NMR)
+			invert_axis = true;
+		break;
+	default:
+		break;
+	}
+	if (m_XAxisInvertBtn) {
+		g_signal_handler_block (m_XAxisInvertBtn, m_XAxisInvertSgn);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_XAxisInvertBtn), invert_axis);
+		g_signal_handler_unblock (m_XAxisInvertBtn, m_XAxisInvertSgn);
+	}
+	m_View->SetAxisBounds (GOG_AXIS_X, minx, maxx, invert_axis);
+	m_View->SetAxisBounds (GOG_AXIS_Y, miny, maxy, false);
+	if (hide_y_axis)
+		m_View->ShowAxis (GOG_AXIS_Y, false);
+	/* Add axes labels */
+	if (m_XUnit < GCU_SPECTRUM_UNIT_MAX)
+		m_View->SetAxisLabel (GOG_AXIS_X, _(UnitNames[m_XUnit]));
+	if (m_YUnit < GCU_SPECTRUM_UNIT_MAX)
+		m_View->SetAxisLabel (GOG_AXIS_Y, _(UnitNames[m_YUnit]));
+	return true;
+}
+
+bool SpectrumDocument::SetProperty (unsigned property, char const *value)
+{
+	istringstream is (value);
+	switch (property) {
+	case GCU_PROP_DOC_TITLE:
+		SetTitle (value);
+		break;
+	case GCU_PROP_SPECTRUM_TYPE:
+		m_SpectrumType = get_spectrum_type_from_string (value);
+		break;
+	case GCU_PROP_SPECTRUM_NPOINTS:
+		is >> npoints;
+		break;
+	case GCU_PROP_SPECTRUM_DATA_X:
+		break;
+	case GCU_PROP_SPECTRUM_DATA_Y:
+		break;
+	case GCU_PROP_SPECTRUM_DATA_REAL: {
+		if (npoints == 0 || R >= 0)
+			return false;
+		JdxVar vr;
+		vr.Name = _("Real data");
+		vr.Symbol = 'r';
+		vr.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
+		vr.Unit = GCU_SPECTRUM_UNIT_MAX;
+		vr.Format = GCU_SPECTRUM_FORMAT_MAX;
+		vr.Factor = 1.;
+		vr.NbValues = npoints;
+		vr.Values = new double[npoints];
+		for (unsigned i = 0; i < npoints; i++)
+			is >> vr.Values[i];
+		vr.First = vr.Values[0];
+		vr.Last = vr.Values[npoints - 1];
+		go_range_min (vr.Values, npoints, &vr.Min);
+		go_range_max (vr.Values, npoints, &vr.Max);
+		vr.Series = NULL;
+		Y = R = variables.size ();
+		variables.push_back (vr);
+		break;
+	}
+	case GCU_PROP_SPECTRUM_DATA_IMAGINARY: {
+		if (npoints == 0 || I >= 0)
+			return false;
+		JdxVar vi;
+		vi.Name = _("Imaginary data");
+		vi.Symbol = 'i';
+		vi.Type = GCU_SPECTRUM_TYPE_DEPENDENT;
+		vi.Unit = GCU_SPECTRUM_UNIT_MAX;
+		vi.Format = GCU_SPECTRUM_FORMAT_MAX;
+		vi.Factor = 1.;
+		vi.NbValues = npoints;
+		vi.Values = new double[npoints];
+		for (unsigned i = 0; i < npoints; i++)
+			is >> vi.Values[i];
+		vi.First = vi.Values[0];
+		vi.Last = vi.Values[npoints - 1];
+		go_range_min (vi.Values, npoints, &vi.Min);
+		go_range_max (vi.Values, npoints, &vi.Max);
+		vi.Series = NULL;
+		I = variables.size ();
+		variables.push_back (vi);
+		break;
+	}
+	case GCU_PROP_SPECTRUM_X_UNIT:
+			m_XUnit = (SpectrumUnitType) get_spectrum_data_from_string (value, Units, GCU_SPECTRUM_UNIT_MAX);
+		break;
+	case GCU_PROP_SPECTRUM_X_MIN:
+		is >> minx;
+		break;
+	case GCU_PROP_SPECTRUM_X_MAX:
+		is >> maxx;
+		break;
+	case GCU_PROP_SPECTRUM_X_OFFSET:
+		is >> offset;
+		break;
+	case GCU_PROP_SPECTRUM_NMR_FREQ:
+		is >> freq;
+		break;
+	default:
+		return false;
+	}
+	return true;
+}
+
 }	//	nampespace gcu
