@@ -330,6 +330,18 @@ Called to transform an FID to the NMR spectrum. The label in the GtkButton
 might be changed.
 */
 	void OnTransformFID (GtkButton *btn);
+/*!
+@param property the property id as defined in objprops.h
+@param value the property value as a string
+
+Used when loading to set properties to spectra.
+@return true if the property could be set, or if the property is not relevant, false otherwise.
+*/
+	bool SetProperty (unsigned property, char const *value);
+/*!
+Called by the application when the document has been loaded.
+*/
+	bool Loaded () throw (gcu::LoaderError);
 
 private:
 	void LoadJcampDx (char const *data);
@@ -346,7 +358,7 @@ private:
 	double firstx, lastx, deltax, firsty;
 	double xfactor, yfactor;
 	std::vector <JdxVar> variables;
-	int X, Y, R, I, integral, Rt, It, Rp;
+	int X, Xt, Y, R, I, integral, Rt, It, Rp;
 	double freq;
 	double offset, refpoint;
 	GtkWidget *m_XAxisInvertBtn;
