@@ -338,7 +338,7 @@ int main (int argc, char *argv[])
 	gtk_init (&argc, &argv);
 	libgoffice_init ();
 	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_TYPE_PLUGIN_LOADER_MODULE);
-			Loader::Init ();
+	Loader::Init (Application::GetDefaultApplication ());
 	in_channel = g_io_channel_unix_new (fileno (stdin));
 	g_io_add_watch (in_channel, G_IO_IN, io_func, &error);
 	gtk_main ();
