@@ -32,6 +32,7 @@
 #include <gcu/loader.h>
 #include <gcu/molecule.h>
 #include <gcu/objprops.h>
+#include <gcu/xml-utils.h>
 
 #include <goffice/app/module-plugin-defs.h>
 #include <gsf/gsf-libxml.h>
@@ -46,6 +47,7 @@
 #include <cstring>
 
 #include <iostream>
+#include <sstream>
 
 using namespace std;
 using namespace gcu;
@@ -123,6 +125,7 @@ typedef struct {
 
 typedef struct {
 	Document *doc;
+	Application *app;
 	GOIOContext *context;
 	stack<Object*> cur;
 	list<CDXMLProps> failed;
@@ -161,8 +164,9 @@ static void
 cdxml_fragment_start (GsfXMLIn *xin, G_GNUC_UNUSED xmlChar const **attrs)
 {
 	CDXMLReadState	*state = (CDXMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("molecule", state->cur.top ());
+	Object *obj = state->app->CreateObject ("molecule", state->cur.top ());
 	state->cur.push (obj);
+	state->doc->ObjectLoaded (obj);
 }
 
 static void
@@ -180,7 +184,7 @@ static void
 cdxml_bond_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	CDXMLReadState	*state = (CDXMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("bond", state->cur.top ());
+	Object *obj = state->app->CreateObject ("bond", state->cur.top ());
 	obj->SetProperty (GCU_PROP_BOND_ORDER, "1");
 	map<string, unsigned>::iterator it;
 	while (*attrs) {
@@ -251,14 +255,16 @@ cdxml_bond_start (GsfXMLIn *xin, xmlChar const **attrs)
 		attrs++;
 	}
 	state->cur.push (obj);
+	state->doc->ObjectLoaded (obj);
 }
 
 static void
 cdxml_text_start (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	CDXMLReadState	*state = (CDXMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("text", state->cur.top ());
+	Object *obj = state->app->CreateObject ("text", state->cur.top ());
 	state->cur.push (obj);
+	state->doc->ObjectLoaded (obj);
 	char *lowered;
 	map<string, unsigned>::iterator it;
 	while (*attrs)
@@ -335,8 +341,9 @@ cdxml_string_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 		// for now put all numbers as subscripts
 		// FIXME: fix this kludgy code
 		int cur = 0, size = strlen (xin->content->str);
-		char *new_size = g_strdup_printf ("%g", strtod (state->size.c_str (), NULL) / 1.5);
-		char *height = g_strdup_printf ("%g", strtod (state->size.c_str (), NULL) / 3.);
+		char new_size[G_ASCII_DTOSTR_BUF_SIZE], height[G_ASCII_DTOSTR_BUF_SIZE];
+		g_ascii_dtostr (new_size, G_ASCII_DTOSTR_BUF_SIZE, g_ascii_strtod (state->size.c_str (), NULL) / 1.5);
+		g_ascii_dtostr (height, G_ASCII_DTOSTR_BUF_SIZE, g_ascii_strtod (state->size.c_str (), NULL) / 3.);
 		while (cur < size) {
 			while (cur < size && (xin->content->str[cur] < '0' || xin->content->str[cur] > '9'))
 				state->markup += xin->content->str[cur++];
@@ -373,8 +380,6 @@ cdxml_string_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 					opened = false;
 			}
 		}
-		g_free (new_size);
-		g_free (height);
 	} else {
 		state->markup += xin->content->str;
 		if (state->attributes & 0x20)
@@ -504,7 +509,7 @@ fragment_success:
 			mol = reinterpret_cast <Molecule *> (state->cur.top ());
 			mol->Remove (atom);
 			delete atom;
-			atom = Object::CreateObject ("fragment", mol);
+			atom = state->app->CreateObject ("fragment", mol);
 			atom->SetProperty (GCU_PROP_TEXT_TEXT, buf.c_str ());
 			atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, id.c_str ());
 			atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
@@ -535,8 +540,9 @@ cdxml_node_start (GsfXMLIn *xin, xmlChar const **attrs)
 	GSF_XML_IN_NODE_END
 	};
 	CDXMLReadState	*state = (CDXMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("atom", state->cur.top ());
+	Object *obj = state->app->CreateObject ("atom", state->cur.top ());
 	obj->SetProperty (GCU_PROP_ATOM_Z, "6");
+	state->doc->ObjectLoaded (obj);
 	map<string, unsigned>::iterator it;
 	bool fragment = false;
 	while (*attrs) {
@@ -557,7 +563,7 @@ cdxml_node_start (GsfXMLIn *xin, xmlChar const **attrs)
 				if (mol)
 					mol->Remove (obj);
 				delete obj;
-				obj = Object::CreateObject ("pseudo-atom", state->cur.top ());
+				obj = state->app->CreateObject ("pseudo-atom", state->cur.top ());
 				if (id.length ())
 					obj->SetProperty (GCU_PROP_ID, id.c_str ());
 				obj->SetProperty (GCU_PROP_POS2D, pos.c_str ());
@@ -572,6 +578,7 @@ cdxml_node_start (GsfXMLIn *xin, xmlChar const **attrs)
 		if (NULL == doc)
 			doc = gsf_xml_in_doc_new (atom_dtd, NULL);
 		state->cur.push (obj); // push it twice in that case
+		state->doc->ObjectLoaded (obj);
 		gsf_xml_in_push_state (xin, doc, state, (GsfXMLInExtDtor) fragment_done, attrs);
 	}
 }
@@ -615,9 +622,10 @@ static void
 cdxml_group_start (GsfXMLIn *xin, G_GNUC_UNUSED xmlChar const **attrs)
 {
 	CDXMLReadState	*state = (CDXMLReadState *) xin->user_state;
-	Object *obj = Object::CreateObject ("group", state->cur.top ());
+	Object *obj = state->app->CreateObject ("group", state->cur.top ());
 	obj->Lock ();
 	state->cur.push (obj);
+	state->doc->ObjectLoaded (obj);
 }
 
 static void
@@ -630,9 +638,10 @@ cdxml_graphic_start (GsfXMLIn *xin, xmlChar const **attrs)
 	while (*attrs) {
 		if (!strcmp ((char const *) *attrs, "id"))
 			Id = atoi ((char const *) attrs[1]);
-		else if (!strcmp ((char const *) *attrs, "BoundingBox"))
-			sscanf ((char const *) attrs[1], "%lg %lg %lg %lg", &x1, &y1, &x0, &y0);
-		else if (!strcmp ((char const *) *attrs, "GraphicType")) {
+		else if (!strcmp ((char const *) *attrs, "BoundingBox")) {
+			istringstream str (reinterpret_cast <char const *> (attrs[1]));
+			str >> x1 >> y1 >> x0 >> y0;
+		} else if (!strcmp ((char const *) *attrs, "GraphicType")) {
 			if (!strcmp ((char const *) attrs[1], "Line"))
 				type = 1;
 		} else if (!strcmp ((char const *) *attrs, "ArrowType")) {
@@ -650,36 +659,36 @@ cdxml_graphic_start (GsfXMLIn *xin, xmlChar const **attrs)
 		attrs+=2;
 	}
 	if (type == 1) {
-		char *buf = NULL;
 		Object *obj = NULL;
+		ostringstream str;
 		switch (arrow_type) {
 		case 1:
 		case 2:
-			obj = Object::CreateObject ("reaction-arrow", state->cur.top ());
-			buf = g_strdup_printf ("ra%d", Id);
+			obj = state->app->CreateObject ("reaction-arrow", state->cur.top ());
+			str << "ra" << Id;
 			break;
 		case 4:
-			obj = Object::CreateObject ("mesomery-arrow", state->cur.top ());
-			buf = g_strdup_printf ("ma%d", Id);
+			obj = state->app->CreateObject ("mesomery-arrow", state->cur.top ());
+			str << "ma" << Id;
 			break;
 		case 8:
-			obj = Object::CreateObject ("reaction-arrow", state->cur.top ());
-			buf = g_strdup_printf ("ra%d", Id);
+			obj = state->app->CreateObject ("reaction-arrow", state->cur.top ());
+			str << "ra" << Id;
 			obj->SetProperty (GCU_PROP_REACTION_ARROW_TYPE, "double");
 			break;
 		case 32:
-			obj = Object::CreateObject ("retrosynthesis-arrow", state->cur.top ());
-			buf = g_strdup_printf ("rsa%d", Id);
+			obj = state->app->CreateObject ("retrosynthesis-arrow", state->cur.top ());
+			str << "rsa" << Id;
 			break;
 		default:
 			break;
 		}
 		if (obj) {
-			obj->SetId (buf);
-			g_free (buf);
-			buf = g_strdup_printf ("%g %g %g %g", x0, y0, x1, y1);
-			obj->SetProperty (GCU_PROP_ARROW_COORDS, buf);
-			g_free (buf);
+			obj->SetId (str.str ().c_str ());
+			ostringstream str_;
+			str_ << x0 << " " << y0 << " " << x1 << " " << y1;
+			obj->SetProperty (GCU_PROP_ARROW_COORDS, str_.str ().c_str ());
+			state->doc->ObjectLoaded (obj);
 		}
 	}
 }
@@ -738,6 +747,7 @@ ContentType CDXMLLoader::Read  (Document *doc, GsfInput *in, G_GNUC_UNUSED char
 	CDXMLReadState state;
 
 	state.doc = doc;
+	state.app = doc->GetApplication ();
 	state.context = io;
 	ContentType success = ContentTypeUnknown;
 	state.colors.push_back ("red=\"1\" green=\"1\" blue=\"1\""); // white
@@ -892,13 +902,6 @@ bool CDXMLLoader::WriteObject (xmlDocPtr xml, xmlNodePtr node, Object *object, G
 					either in this code or in the cml schema */
 }
 
-void CDXMLLoader::AddIntProperty (xmlNodePtr node, char const *id, int value)
-{
-	char *buf = g_strdup_printf("%d", value);
-	xmlNewProp (node, reinterpret_cast <xmlChar const *> (id), reinterpret_cast <xmlChar const *> (buf));
-	g_free (buf);
-}
-
 void CDXMLLoader::AddStringProperty (xmlNodePtr node, char const *id, string &value)
 {
 	xmlNewProp (node, reinterpret_cast <xmlChar const *> (id), reinterpret_cast <xmlChar const *> (value.c_str ()));
@@ -967,22 +970,16 @@ bool CDXMLLoader::Write  (Object *obj, GsfOutput *out, G_GNUC_UNUSED char const
 	for (color = m_Colors.begin (); color != end_color; color++) {
 		xmlNodePtr node = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("color"), NULL);
 		xmlAddChild (colors, node);
-		char *buf = g_strdup_printf ("%g", GO_COLOR_DOUBLE_R ((*color).second));
-		xmlNewProp (node, reinterpret_cast <xmlChar const *> ("r"), reinterpret_cast <xmlChar const *> (buf));
-		g_free (buf);
-		buf = g_strdup_printf ("%g", GO_COLOR_DOUBLE_G ((*color).second));
-		xmlNewProp (node, reinterpret_cast <xmlChar const *> ("g"), reinterpret_cast <xmlChar const *> (buf));
-		g_free (buf);
-		buf = g_strdup_printf ("%g", GO_COLOR_DOUBLE_B ((*color).second));
-		xmlNewProp (node, reinterpret_cast <xmlChar const *> ("b"), reinterpret_cast <xmlChar const *> (buf));
-		g_free (buf);
+		WriteFloat (node, "r", GO_COLOR_DOUBLE_R ((*color).second));
+		WriteFloat (node, "g", GO_COLOR_DOUBLE_G ((*color).second));
+		WriteFloat (node, "b", GO_COLOR_DOUBLE_B ((*color).second));
 	}
 	// write fonts
 	map <unsigned, CDXMLFont>::iterator font, end_font = m_Fonts.end ();
 	for (font = m_Fonts.begin (); font != end_font; font++) {
 		xmlNodePtr node = xmlNewDocNode (xml, NULL, reinterpret_cast <xmlChar const *> ("font"), NULL);
 		xmlAddChild (fonttable, node);
-		AddIntProperty (node, "id", (*font).second.index);
+		WriteInt (node, "id", (*font).second.index);
 		AddStringProperty (node, "charset", (*font).second.encoding);
 		AddStringProperty (node, "name", (*font).second.name);
 	}
