@@ -45,35 +45,6 @@ static Object* CreatePseudoAtom ()
 
 gcpResiduesPlugin::gcpResiduesPlugin (): gcp::Plugin ()
 {
-	PseudoAtomType = Object::AddType ("pseudo-atom", CreatePseudoAtom);
-	xmlDocPtr doc;
-	xmlNodePtr node;
-	char *name;
-	xmlIndentTreeOutput = true;
-	xmlKeepBlanksDefault (0);
-	doc = xmlParseFile (PKGDATADIR"/residues.xml");
-	if (doc) {
-		docs.insert (doc);
-		node = doc->children;
-		if (!strcmp ((char*)node->name, "residues"))
-			ParseNodes (node->children, false);
-	}
-	char* gcupath = g_strconcat (getenv ("HOME"), "/.gchemutils", NULL);
-	GDir* dir = g_dir_open (gcupath, 0, NULL);
-	if (dir)
-		g_dir_close (dir);
-	else
-		mkdir (gcupath, 0x1ed);
-	g_free (gcupath) ;
-	name = g_strconcat (getenv ("HOME"), "/.gchemutils/residues.xml", NULL);
-	if (g_file_test (name, G_FILE_TEST_EXISTS) && (doc = xmlParseFile (name))) {
-		docs.insert (doc);
-		user_residues = doc;
-		node = doc->children;
-		if (!strcmp ((char*)node->name, "residues"))
-			ParseNodes (node->children, true);
-	}
-	g_free (name);
 }
 
 gcpResiduesPlugin::~gcpResiduesPlugin ()
@@ -130,6 +101,35 @@ static void on_menu (GtkUIManager *UIManager)
 void gcpResiduesPlugin::Populate (gcp::Application *App)
 {
 	m_App = App;
+	PseudoAtomType = App->AddType ("pseudo-atom", CreatePseudoAtom);
+	xmlDocPtr doc;
+	xmlNodePtr node;
+	char *name;
+	xmlIndentTreeOutput = true;
+	xmlKeepBlanksDefault (0);
+	doc = xmlParseFile (PKGDATADIR"/residues.xml");
+	if (doc) {
+		docs.insert (doc);
+		node = doc->children;
+		if (!strcmp ((char*)node->name, "residues"))
+			ParseNodes (node->children, false);
+	}
+	char* gcupath = g_strconcat (getenv ("HOME"), "/.gchemutils", NULL);
+	GDir* dir = g_dir_open (gcupath, 0, NULL);
+	if (dir)
+		g_dir_close (dir);
+	else
+		mkdir (gcupath, 0x1ed);
+	g_free (gcupath) ;
+	name = g_strconcat (getenv ("HOME"), "/.gchemutils/residues.xml", NULL);
+	if (g_file_test (name, G_FILE_TEST_EXISTS) && (doc = xmlParseFile (name))) {
+		docs.insert (doc);
+		user_residues = doc;
+		node = doc->children;
+		if (!strcmp ((char*)node->name, "residues"))
+			ParseNodes (node->children, true);
+	}
+	g_free (name);
 	App->RegisterOptions (options);
 	App->AddMenuCallback (on_menu);
 	gcp::Residue::SetPostAddCallback (on_new_residue);
@@ -150,7 +150,7 @@ void gcpResiduesPlugin::ParseNodes (xmlNodePtr node, bool writeable)
 	while (node) {
 		if (!strcmp ((char*) node->name, "residue")) {
 			r = new gcp::Residue ();
-			r->Load (node, !writeable);
+			r->Load (node, !writeable, m_App);
 		}
 		node = node->next;
 	}
@@ -198,7 +198,7 @@ void gcpResiduesPlugin::OnNewResidue (gcp::Residue *res)
 			xmlKeepBlanksDefault (0);
 			xmlSaveFormatFile ((char*) user_residues->URL, user_residues, true);
 			xmlFreeDoc (xml);
-			res->Load (node, false);
+			res->Load (node, false, m_App);
 		}
 	}
 	if (dlg)
