@@ -103,11 +103,11 @@ GChemCalc::GChemCalc ():
 	Application ("gchemcalc"),
 	formula ("")
 {
-	Object::AddType ("atom", CreateAtom, AtomType);
-	Object::AddType ("pseudo-atom", CreateAtom);
-	Object::AddType ("bond", CreateBond, BondType);
-	Object::AddType ("molecule", CreateMolecule, MoleculeType);
-	Object::AddType ("document", CreateDocument, DocumentType);
+	AddType ("atom", CreateAtom, AtomType);
+	AddType ("pseudo-atom", CreateAtom);
+	AddType ("bond", CreateBond, BondType);
+	AddType ("molecule", CreateMolecule, MoleculeType);
+	AddType ("document", CreateDocument, DocumentType);
 	// Load residues
 	xmlDocPtr doc;
 	char *name;
@@ -137,7 +137,7 @@ void GChemCalc::ParseNodes (xmlNodePtr node)
 	while (node) {
 		if (!strcmp ((char*) node->name, "residue")) {
 			r = new Residue ();
-			r->Load (node);
+			r->Load (node, this);
 		}
 		node = node->next;
 	}
@@ -581,7 +581,7 @@ static void on_mode (GtkRadioAction *action, G_GNUC_UNUSED GtkRadioAction *curre
 	App->formula.SetParseMode (static_cast <FormulaParseMode> (gtk_radio_action_get_current_value (action)));
 }
 
-static void on_page (G_GNUC_UNUSED GtkNotebook *book, G_GNUC_UNUSED void *p, int page)
+static void on_page (G_GNUC_UNUSED GtkNotebook *book, G_GNUC_UNUSED GtkNotebookPage *p, int page)
 {
 	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/SaveAsImage"), page);
 	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/PageSetup"), page);
